@@ -1,6 +1,6 @@
 ;;; (json builder) --- Guile JSON implementation.
 
-;; Copyright (C) 2013-2018 Aleix Conchillo Flaque <aconchillo@gmail.com>
+;; Copyright (C) 2013-2019 Aleix Conchillo Flaque <aconchillo@gmail.com>
 ;; Copyright (C) 2015,2016 Jan Nieuwenhuizen <janneke@gnu.org>
 ;;
 ;; This file is part of guile-json.
@@ -80,7 +80,7 @@
          (unicode->string (+ (ash (bytevector-u8-ref bv4 2) 8)
                              (bytevector-u8-ref bv4 3))))))
      ;; Anything else should wrong, hopefully.
-     (else (throw 'json-invalid)))))
+     (else (throw 'json-invalid (string c))))))
 
 ;;
 ;; Object builder functions
@@ -124,9 +124,6 @@
         ((symbol? x) (symbol->string x))
         (else x)))
 
-(define (atom? x)
-  (or (char? x) (number? x) (string? x) (symbol? x)))
-
 (define (json-build-string scm port escape unicode)
   (display "\"" port)
   (display
@@ -173,16 +170,40 @@
   (build-newline port pretty)
   (simple-format port "~A}" (indent-string pretty level)))
 
+(define (json-number? number)
+  (and (number? number) (eqv? (imag-part number) 0) (finite? number)))
+
 (define (json-build scm port escape unicode pretty level)
   (cond
    ((eq? scm #nil) (json-build-null port))
    ((boolean? scm) (json-build-boolean scm port))
-   ((number? scm) (json-build-number scm port))
+   ((json-number? scm) (json-build-number scm port))
    ((symbol? scm) (json-build-string (symbol->string scm) port escape unicode))
    ((string? scm) (json-build-string scm port escape unicode))
    ((vector? scm) (json-build-array scm port escape unicode pretty level))
-   ((pair? scm) (json-build-object scm port escape unicode pretty level))
-   (else (throw 'json-invalid))))
+   ((or (pair? scm) (null? scm))
+    (json-build-object scm port escape unicode pretty level))
+   (else (throw 'json-invalid scm))))
+
+(define (json-key? scm)
+  (or (symbol? scm) (string? scm)))
+
+(define (json-valid? scm)
+  (cond
+   ((eq? scm #nil) #t)
+   ((boolean? scm) #t)
+   ((json-number? scm) #t)
+   ((symbol? scm) #t)
+   ((string? scm) #t)
+   ((vector? scm) (vector-every json-valid? scm))
+   ((pair? scm)
+    (every (lambda (entry)
+	     (and (pair? entry)
+		  (json-key? (car entry))
+		  (json-valid? (cdr entry))))
+	   scm))
+   ((null? scm) #t)
+   (else (throw 'json-invalid scm))))
 
 ;;
 ;; Public procedures
@@ -190,18 +211,33 @@
 
 (define* (scm->json scm
                     #:optional (port (current-output-port))
-                    #:key (escape #f) (unicode #f) (pretty #f))
-  "Creates a JSON document from native. The argument @var{scm} contains
-the native value of the JSON document. Takes one optional argument,
-@var{port}, which defaults to the current output port where the JSON
-document will be written."
-  (json-build scm port escape unicode pretty 0))
+                    #:key (escape #f) (unicode #f) (pretty #f) (validate #t))
+  "Creates a JSON document from native. The argument @var{scm} contains the
+native value of the JSON document. Takes one optional argument, @var{port},
+which defaults to the current output port where the JSON document will be
+written. It also takes a few key arguments: @var{escape}: if true, the
+slash (/ solidus) character will be escaped, @{unicode} : if true, unicode
+characters will be escaped when needed and @{pretty}: if true, the JSON
+document will be pretty printed.
 
-(define* (scm->json-string scm #:key (escape #f) (unicode #f) (pretty #f))
-  "Creates a JSON document from native into a string. The argument
-@var{scm} contains the native value of the JSON document."
+Note that when using alists to build JSON objects, symbols or numbers might be
+used as keys and they both will be converted to strings.
+"
+  (cond
+   ((and validate (json-valid? scm))
+    (json-build scm port escape unicode pretty 0))
+   (else
+    (json-build scm port escape unicode pretty 0))))
+
+(define* (scm->json-string scm #:key
+                           (escape #f) (unicode #f)
+                           (pretty #f) (validate #f))
+  "Creates a JSON document from native into a string. The argument @var{scm}
+contains the native value of the JSON document."
   (call-with-output-string
    (lambda (p)
-     (scm->json scm p #:escape escape #:unicode unicode #:pretty pretty))))
+     (scm->json scm p
+                #:escape escape #:unicode unicode
+                #:pretty pretty #:validate validate))))
 
 ;;; (json builder) ends here
